@@ -7,7 +7,7 @@
 #############################################################################
 
 #Loading required libraries:
-library(astro)
+library(magicaxis)
 library(astroFns)
 library(plotrix)
 library(raster)
@@ -17,28 +17,35 @@ library(DescTools)
 library(PBSmapping)
 library(R.utils)
 library(rgeos)
+library(yaml)
 
-#Probe shape definition as global variables (currently has a head/tip, a body/probe and a cable zone of avoidance (all should be in degrees):
+#Probe shape definition as global variables (currently has a head/tip, a body/probe and a cable zone of avoidance (all should be in mm):
 #plate_scale"/mm scaling
-plate_scale <<- 15.008
+#plate_scale <<- 15.008
+#! Added by Sam to use the same constants file as Ayoan's code uses
+#hector_constants <<- read_yaml(paste(Sys.getenv('HECTOROBSPIPELINE_LOC'),"constants/HECTOR_CONSTANTS.yaml", sep='/'))
+hector_constants <<- read_yaml("/Users/samvaughan/Science/Hector/HectorObservationPipeline/hop/constants/HECTOR_CONSTANTS.yaml")
+
+
 
 #Field dimensions and parameters:
-fov <<- 236*2 * plate_scale/3600 #2 degree field - a bit for sky fibres.
-wceg <<- 2.* 15 * plate_scale/3600 #Width of the cable exit gap (this is the 2*n where n=15mm initially in Julia's notes)
-ceg_pos <<- cbind(x=c(0,0.8504,-0.8504),y=c(-0.982,0.49098,0.49098), deg=c(-90,30,150))
-mid_ceg_pos<<-cbind(x=c(0,-0.8504,0.8504),y=c(0.982,-0.49098,-0.49098), deg= c(90,210,-30))
-E0<<-sqrt((wceg/2.)**2-(9*plate_scale/3600)**2) #E is the line joining the solid ferule end to the centre of the exit point. 
+fov <<- hector_constants$HECTOR_plate_radius*2  #diameter of the field in mm
+skybuffer<<-8 #clear-edge buffer
+wceg <<- 2.* 15  #Width of the cable exit gap (this is the 2*n where n=15mm initially in Julia's notes)
+ceg_pos <<- cbind(x=fov/2.*cosd(c(-90,30,150)),y=fov/2.*sind(c(-90,30,150)), deg=c(-90,30,150))
+E0<<-sqrt((wceg/2.)**2-(9)**2) #E is the line joining the solid ferule end to the centre of the exit point. 
 #E0=SQRT((n^2)-81). If E<E0, then need to point to the next least contaminated exit. This step prevents a ferule from 
 #blocking an exit such that other cables will not get past.
-fbr <<- 45 * plate_scale/3600 #Fibre bend radius
+fbr <<- 45 #Fibre bend radius
 
 #Galaxy and standard star bundle dimensions:
-tip_w <<- 14.5/sqrt(2) * plate_scale/3600 #(instead of 12)
-tip_l <<- 14.5/sqrt(2) * plate_scale/3600  
-probe_w <<- 14.5 * plate_scale/3600 #(instead of 8.5)
-probe_l <<- 45 * plate_scale/3600
-cable_w <<- 8.5 * plate_scale/3600
-cable_l <<- 38 * plate_scale/3600
+tip_w <<- 14.5/sqrt(2)  
+tip_l <<- 14.5/sqrt(2) 
+# Testing- make this 18.5 from 14.5
+probe_w <<- 18.5 
+probe_l <<- 45 
+cable_w <<- 8.5 
+cable_l <<- 38 
 excl_radius <<- sqrt(2.*((tip_w/2)^2))
 dngalprobes <<- 19
 nstdprobes <<- 2
@@ -49,17 +56,22 @@ nprobes <<- dngalprobes #Start with the number of galaxy targets. There will be
 
 #Guide bundle dimensions:
 #Julia: The guide bundles should be the same 14mm width but shorter than the hexabundle ferules by 20mm ~50mm total plus cable bit.
-gtip_w <<- 14.5/sqrt(2) * plate_scale/3600
-gtip_l <<- 14.5/sqrt(2) * plate_scale/3600
-gprobe_w <<- 14.5 * plate_scale/3600
-gprobe_l <<- 45 * plate_scale/3600 
-gcable_w <<- 8.5 * plate_scale/3600
-gcable_l <<- 38 * plate_scale/3600
+gtip_w <<- 14.5/sqrt(2)
+gtip_l <<- 14.5/sqrt(2)
+gprobe_w <<- 14.5
+gprobe_l <<- 45
+gcable_w <<- 8.5
+gcable_l <<- 38
 gexcl_radius <<- sqrt(2.*((gtip_w/2)^2))
 
 #Setting timeout time (in cpu seconds, I think) for how long each attempt lasts before moving on. 
 #The code will try for at most 15 times that before starting to swap a probe.
-timeout<<-180 #180 seconds x 15 is about half an hour and is about right. Less than about 90 seconds led to problems in highly conflicted field..
+#timeout<<-180 #180 seconds x 15 is about half an hour and is about right. Less than about 90 seconds led to problems in highly conflicted field..
+timeout<<-600 #Increased to account for longer conflict calculation time due to rounded heads so code still has enough time time try enough angles variations.
+timeout_behaviour = 'error'
+
+#angle step going around the head to approximate circle as a polygon. The smaller angles are more precice but lead to longer configuration time.
+delta_poly<<-5 #5 degrees leads to a 0.1% radius buffer around the head and configuration seems to take a bit longer than a square head.
 
 #Compute the (horizontal/RA/x-axis) distance to the field edge.
 dist2fieldedge <- function(x,y){
@@ -189,25 +201,6 @@ adjust_angle <- function(theta,fgang){
   } else if (theta > fgang + 180){
     theta=theta-360
   }
-  
-  #fgang is the first_guess_ang.
-  #Upper left, should use a range that is between -180 and 180.
-  # if (fgang<0){
-  #   if (theta < -180) theta=theta+360
-  #   if (theta > 180) theta=theta-360
-  # #Upper right use a range that is between 90 and 450.
-  # } else if (fgang > 180){
-  #   while (theta <90) theta=theta+360
-  #   if (theta > 450) theta=theta-360
-  # # Lower right
-  # } else if (fgang >90 & fgang<180) {
-  #   while (theta <0) theta=theta+360
-  #   if (theta > 360) theta=theta-360
-  # #Lower left should be between -90 and 270
-  # } else {
-  #   if (theta < -90) theta=theta+360
-  #   if (theta > 270) theta=theta-360
-  # }
   return(theta)
 }
 
@@ -278,9 +271,9 @@ random_allocation <- function (nrandprobes=ngalprobes+6, nstdprobes=20, nguidepr
 #Draw buttons around each position to be allocated.
 draw_pos <- function(x,y, label_probe=TRUE, overwrite=TRUE){
   if (overwrite){ #set overwrite to true to start the plots from scratch.
-    aplot(0,0,pch='x',col="red",xlim=c(-fov/2.,fov/2.),ylim=c(-fov/2.,fov/2.), asp=1, xlab='X (deg)', ylab='Y (deg)')
+    magplot(0,0,pch='x',col="red",xlim=c(-fov/2.,fov/2.),ylim=c(-fov/2.,fov/2.), asp=1, xlab='X (mm)', ylab='Y (mm)')
     draw.circle(0,0,radius=fov/2.,border='red')
-    draw.circle(0,0,radius=1.,border='black')
+    draw.circle(0,0,radius=fov/2.+skybuffer,border='black')
     #Draw the cable exit gaps in cyan
     for (cceg in c(1:length(ceg_pos[,1]))){
       dx=wceg/2.* sind(ceg_pos[cceg,'deg'])
@@ -332,19 +325,20 @@ defineprobe <- function(xs,ys,angs,interactive=FALSE, col='black'){
   for (i in c(1:length(xs))) {
     x=xs[i]
     y=ys[i]
-    probe=cbind(c(x+tip_w/2., 
-                  x-(tip_l-tip_w/2.),
-                  x-(tip_l-tip_w/2.),
-                  x-(tip_l-tip_w/2.)-probe_l, 
-                  x-(tip_l-tip_w/2.)-probe_l, 
-                  x-(tip_l-tip_w/2.)-probe_l-cable_l, 
-                  x-(tip_l-tip_w/2.)-probe_l-cable_l, 
-                  x-(tip_l-tip_w/2.)-probe_l, 
-                  x-(tip_l-tip_w/2.)-probe_l, 
-                  x-(tip_l-tip_w/2.), 
-                  x-(tip_l-tip_w/2.), 
-                  x+tip_w/2.),
-                c(y+tip_w/2., 
+    rbuff=2-cosd(delta_poly/2)#buffer to account for the approximation of small angles difference in radius.
+    #probe includes the whole exclusion area, so including the exclusion radius around the probe head.
+    probe=cbind(c(x+rbuff*excl_radius*sind(seq(-1.*atand(tip_l/tip_w)+delta_poly,180+atand(tip_l/tip_w)-delta_poly,delta_poly)), 
+                  x-tip_l/2.,
+                  x-tip_l/2.,
+                  x-tip_l/2.-probe_l, 
+                  x-tip_l/2.-probe_l, 
+                  x-tip_l/2.-probe_l-cable_l, 
+                  x-tip_l/2.-probe_l-cable_l, 
+                  x-tip_l/2.-probe_l, 
+                  x-tip_l/2.-probe_l, 
+                  x-tip_l/2., 
+                  x-tip_l/2.),
+                c(y-rbuff*excl_radius*cosd(seq(-1.*atand(tip_l/tip_w)+delta_poly,180+atand(tip_l/tip_w)-delta_poly,delta_poly)), 
                   y+tip_w/2., 
                   y+probe_w/2., 
                   y+probe_w/2., 
@@ -354,12 +348,26 @@ defineprobe <- function(xs,ys,angs,interactive=FALSE, col='black'){
                   y-cable_w/2.,
                   y-probe_w/2., 
                   y-probe_w/2., 
-                  y-tip_w/2., 
                   y-tip_w/2.))
+    #footprint is just for display purposes when plotting, it is not used outside this function.
+    lpoly=length(probe[,1])
+    footprint=cbind(c(x+tip_w/2., 
+                      probe[(lpoly-9):lpoly,1], 
+                      x+tip_w/2.),
+                    c(y+tip_w/2., 
+                      probe[(lpoly-9):lpoly,2],
+                      y-tip_w/2.))
+    
+    #Rotating probe.
     probetemp=cbind(Rotate(probe,theta=angs[i], mx=x, my=y)$x, Rotate(probe,theta=angs[i], mx=x, my=y)$y)
     probe=probetemp
     colnames(probe)=c(paste('X',as.character(i), sep=''),paste('Y',as.character(i),sep=''))
-    if (interactive) {polygon(probe, border=col)}
+    
+    footprintemp=cbind(Rotate(footprint,theta=angs[i], mx=x, my=y)$x, Rotate(footprint,theta=angs[i], mx=x, my=y)$y)
+    footprint=footprintemp
+    colnames(footprint)=c(paste('X',as.character(i), sep=''),paste('Y',as.character(i),sep=''))
+    
+    if (interactive) {polygon(footprint, border=col)}#Drawing the actual shape of the probe, not the exclusion area.
     assign(paste('probe',as.character(i), sep=''), probe)
     probes=cbind(probes,probe)
   }
@@ -372,7 +380,9 @@ defineguideprobe <- function(gxs,gys,gangs,interactive=FALSE){
   for (i in c(1:length(gxs))) {
     x=gxs[i]
     y=gys[i]
-    gprobe=cbind(c(x+gtip_w/2., 
+    rbuff=2-cosd(delta_poly/2)#buffer to account for the approximation of small angles difference in radius.
+    #gprobe includes the whole exclusion area, so including the exclusion radius around the probe head.
+    gprobe=cbind(c(x+rbuff*gexcl_radius*sind(seq(-1.*atand(gtip_l/gtip_w)+delta_poly,180+atand(gtip_l/gtip_w)-delta_poly,delta_poly)), 
                    x-(gtip_l-gtip_w/2.),
                    x-(gtip_l-gtip_w/2.),
                    x-(gtip_l-gtip_w/2.)-gprobe_l, 
@@ -382,9 +392,8 @@ defineguideprobe <- function(gxs,gys,gangs,interactive=FALSE){
                    x-(gtip_l-gtip_w/2.)-gprobe_l, 
                    x-(gtip_l-gtip_w/2.)-gprobe_l, 
                    x-(gtip_l-gtip_w/2.), 
-                   x-(gtip_l-gtip_w/2.), 
-                   x+gtip_w/2.),
-                 c(y+gtip_w/2., 
+                   x-(gtip_l-gtip_w/2.)),
+                 c(y-rbuff*gexcl_radius*cosd(seq(-1.*atand(gtip_l/gtip_w)+delta_poly,180+atand(gtip_l/gtip_w)-delta_poly, delta_poly)), 
                    y+gtip_w/2., 
                    y+gprobe_w/2., 
                    y+gprobe_w/2., 
@@ -394,12 +403,26 @@ defineguideprobe <- function(gxs,gys,gangs,interactive=FALSE){
                    y-gcable_w/2.,
                    y-gprobe_w/2., 
                    y-gprobe_w/2., 
-                   y-gtip_w/2., 
                    y-gtip_w/2.))
+    
+    #footprint is just for display purposes when plotting, it is not used outside this function.
+    lpoly=length(gprobe[,1])
+    footprint=cbind(c(x+gtip_w/2., 
+                      gprobe[(lpoly-9):lpoly,1], 
+                      x+gtip_w/2.),
+                    c(y+gtip_w/2., 
+                      gprobe[(lpoly-9):lpoly,2],
+                      y-gtip_w/2.))
+    
+    #Rotating probe.
     gprobetemp=cbind(Rotate(gprobe,theta=gangs[i], mx=x, my=y)$x, Rotate(gprobe,theta=gangs[i], mx=x, my=y)$y)
     gprobe=gprobetemp
     colnames(gprobe)=c(paste('X',as.character(i), sep=''),paste('Y',as.character(i),sep=''))
-    if (interactive) {polygon(gprobe, border='darkgrey')}
+    footprintemp=cbind(Rotate(footprint,theta=gangs[i], mx=x, my=y)$x, Rotate(footprint,theta=gangs[i], mx=x, my=y)$y)
+    footprint=footprintemp
+    colnames(footprint)=c(paste('X',as.character(i), sep=''),paste('Y',as.character(i),sep=''))
+    
+    if (interactive) {polygon(footprint, border='darkgrey')} #Drawing the actual shape of the probe, not the exclusion area.
     assign(paste('gprobe',as.character(i), sep=''), gprobe)
     gprobes=cbind(gprobes,gprobe)
   }
@@ -412,14 +435,15 @@ find_probe_conflicts <- function(probes, pos, angs){
   if (dim(pos)[1] == 1) {
     return(probe_conflicts)
   }
+  lpoly=length(probes[,1]) #length of polygon defining probe.
   for (i in c(1:(length(pos[,'x'])-1))) {
-    p1 <- data.frame(PID=rep(i, 12), POS=1:12, X=probes[,paste('X',as.character(i), sep='')], Y=probes[,paste('Y',as.character(i), sep='')])
+    p1 <- data.frame(PID=rep(i, lpoly), POS=(1:lpoly), X=probes[,paste('X',as.character(i), sep='')], Y=probes[,paste('Y',as.character(i), sep='')])
     for (j in c((i+1):length(pos[,'x']))) {
-      p2 <- data.frame(PID=rep(j, 12), POS=1:12, X=probes[,paste('X',as.character(j), sep='')], Y=probes[,paste('Y',as.character(j), sep='')])
+      p2 <- data.frame(PID=rep(j, lpoly), POS=(1:lpoly), X=probes[,paste('X',as.character(j), sep='')], Y=probes[,paste('Y',as.character(j), sep='')])
       if (!is.null(joinPolys(p1,p2))) {
         junction=joinPolys(p1,p2)
         #Compute the percentage area of a probe that overlaps.100% means they overlap perfectly. 0% means no overlap.
-        areapoly=round(polyarea(junction$X,junction$Y)/polyarea(p1$X,p1$Y) * 100, digits=1)
+        areapoly= round(polyarea(junction$X,junction$Y)/polyarea(p1$X,p1$Y) * 100, digits=1)
         probe_conflicts=rbind(probe_conflicts, c(i,j,areapoly))
       }
     }
@@ -434,26 +458,30 @@ find_probe_conflicts <- function(probes, pos, angs){
 #Looking for guide/target/standard conflicts with the target/standard probes fixed.
 find_guide_conflicts <- function(probes, pos, angs, gprobes, gpos, gangs){
   probe_conflicts=data.frame()
+  lpoly=length(probes[,1]) #length of polygon defining probe.
+  glpoly=length(gprobes[,1])
+  print("Identifying clashes with target and/or standard probes")
   #First identify clashes with target/standard probes:
   for (i in c(1:(length(pos[,'x'])))) {
-    p1 <- data.frame(PID=rep(i, 12), POS=1:12, X=probes[,paste('X',as.character(i), sep='')], Y=probes[,paste('Y',as.character(i), sep='')])
+    p1 <- data.frame(PID=rep(i, lpoly), POS=(1:lpoly), X=probes[,paste('X',as.character(i), sep='')], Y=probes[,paste('Y',as.character(i), sep='')])
     for (j in 1:length(gpos[,'x'])) {
-      p2 <- data.frame(PID=rep(j, 12), POS=1:12, X=gprobes[,paste('X',as.character(j), sep='')], Y=gprobes[,paste('Y',as.character(j), sep='')])
+      p2 <- data.frame(PID=rep(j, glpoly), POS=(1:glpoly), X=gprobes[,paste('X',as.character(j), sep='')], Y=gprobes[,paste('Y',as.character(j), sep='')])
       if (!is.null(joinPolys(p1,p2))) {
         junction=joinPolys(p1,p2)
         #Compute the percentage area of a probe that overlaps.100% means they overlap perfectly. 0% means no overlap.
-        areapoly=round(polyarea(junction$X,junction$Y)/polyarea(p1$X,p1$Y) * 100, digits=1)
+        areapoly= round(polyarea(junction$X,junction$Y)/polyarea(p1$X,p1$Y) * 100, digits=1)
         probe_conflicts=rbind(probe_conflicts, c(i,j,areapoly))
       }
     }
   }
+  print('Identifying conflict with other guide probes.')
   #Now identify clashes between guide probes.
   if (length(gpos[,'x']) > 1) { #Only if there is more than one guide probe.
     for (i in c(1:(length(gpos[,'x'])))) {
-      p1 <- data.frame(PID=rep(i, 12), POS=1:12, X=gprobes[,paste('X',as.character(i), sep='')], Y=gprobes[,paste('Y',as.character(i), sep='')])
+      p1 <- data.frame(PID=rep(i, lpoly), POS=(1:glpoly), X=gprobes[,paste('X',as.character(i), sep='')], Y=gprobes[,paste('Y',as.character(i), sep='')])
       for (j in c(1:length(gpos[,'x']))) { 
         if (j!=i){
-          p2 <- data.frame(PID=rep(j, 12), POS=1:12, X=gprobes[,paste('X',as.character(j), sep='')], Y=gprobes[,paste('Y',as.character(j), sep='')])
+          p2 <- data.frame(PID=rep(j, lpoly), POS=(1:glpoly), X=gprobes[,paste('X',as.character(j), sep='')], Y=gprobes[,paste('Y',as.character(j), sep='')])
           if (!is.null(joinPolys(p1,p2))) {
             junction=joinPolys(p1,p2)
             #Compute the percentage area of a probe that overlaps.100% means they overlap perfectly. 0% means no overlap.
@@ -485,15 +513,11 @@ choose_cegs <- function(pos) {
   #y=pos[1,'y']
   #draw_pos(x=pos_master$pos[c(1:19),'x'],y=pos_master$pos[c(1:19),'y'])
   
-  #lines(x=c(x,mid_ceg_pos[2,'x']), y=c(y,mid_ceg_pos[2,'y']))
-  #lines(x=c(x,mid_ceg_pos[1,'x']), y=c(y,mid_ceg_pos[1,'y']))
-  #lines(x=c(x,mid_ceg_pos[3,'x']), y=c(y,mid_ceg_pos[3,'y']))
-  
   for (j in c(1:npos)){
     #Define area or "pizza slice" for each cable exit gap.
-    pizza1 = data.frame(PID=rep(1, 122), POS=1:122, X=c(pos[j,'x'],cosd(c(-30:-150))), Y=c(pos[j,'y'],sind(c(-30:-150))))
-    pizza2 = data.frame(PID=rep(2, 122), POS=1:122, X=c(pos[j,'x'],cosd(c(-30:90))), Y=c(pos[j,'y'],sind(c(-30:90))))
-    pizza3 = data.frame(PID=rep(3, 122), POS=1:122, X=c(pos[j,'x'],cosd(c(90:210))), Y=c(pos[j,'y'],sind(c(90:210))))
+    pizza1 = data.frame(PID=rep(1, 122), POS=1:122, X=c(pos[j,'x'],fov/2.*cosd(c(-30:-150))), Y=c(pos[j,'y'],fov/2.*sind(c(-30:-150))))
+    pizza2 = data.frame(PID=rep(2, 122), POS=1:122, X=c(pos[j,'x'],fov/2.*cosd(c(-30:90))), Y=c(pos[j,'y'],fov/2.*sind(c(-30:90))))
+    pizza3 = data.frame(PID=rep(3, 122), POS=1:122, X=c(pos[j,'x'],fov/2.*cosd(c(90:210))), Y=c(pos[j,'y'],fov/2.*sind(c(90:210))))
     nceg=c(-1,-1,-1) #Starting at -1 to avoid counting the probe itself.
     for (i in c(1:npos)){
       cx=pos[i,'x']
@@ -517,9 +541,9 @@ choose_cegs <- function(pos) {
       cegs[j]=which(min(nceg)==nceg)[which.min(pointDistance(p1=ceg_pos[which(min(nceg)==nceg),c('x','y')],p2=c(cx,cy), lonlat=FALSE))]
     }
     #If position is too close to the exit, move them to next closest exit.
-    E=pointDistance(p1=ceg_pos[cegs[j],c(‘x’,‘y’)], p2=pos[j,c(‘x’,‘y’)], lonlat=FALSE) - (probe_l+cable_l/3+tip_l/2.) ##E is the line joining the solid ferule end to the centre of the exit point.
+    E=pointDistance(p1=ceg_pos[cegs[j],c('x','y')], p2=pos[j,c('x','y')], lonlat=FALSE) - (probe_l+cable_l/3+tip_l/2.) ##E is the line joining the solid ferule end to the centre of the exit point.
     if (E < E0){ #E0 is the threshold to avoid blocking a cable exit gap. It is set above.
-      cegs[j]=order(pointDistance(p1=ceg_pos[,c(‘x’,‘y’)],p2=pos[j,c(‘x’,‘y’)], lonlat=FALSE))[2]
+      cegs[j]=order(pointDistance(p1=ceg_pos[,c('x','y')],p2=pos[j,c('x','y')], lonlat=FALSE))[2]
     }
   }
   return(cegs)
@@ -556,9 +580,6 @@ first_guess_angs <- function(pos, cegs, guide=FALSE){
     #}
   #}
   
-  #testing:
-  #draw_pos(x=pos[,'x'],y=pos[,'y'])
-  #defineprobe(xs=pos[,'x'],ys=pos[,'y'],angs=angs, interactive=T)
   return(angs)
 }
 
@@ -642,7 +663,7 @@ wrapMinimizeConflicts <- function(pos=pos, angs=angs, cegs=cegs, init_delta_ang=
     print(paste('Starting (again) with delta_ang =', as.character(delta_ang)))
     
     newangs=angs 
-    withTimeout(assign('newangs',MinimizeConflicts(pos=pos, angs=angs, cegs=cegs, delta_ang=delta_ang, probe_conflicts=conflicts)), timeout=timeout, onTimeout='warning')
+    try(withTimeout(assign('newangs',MinimizeConflicts(pos=pos, angs=angs, cegs=cegs, delta_ang=delta_ang, probe_conflicts=conflicts)), timeout=timeout, onTimeout=timeout_behaviour), silent=TRUE)
     if (FALSE %in% (angs==newangs)){
       angs=newangs
     } else {
@@ -668,7 +689,7 @@ wrapMinimizeConflicts <- function(pos=pos, angs=angs, cegs=cegs, init_delta_ang=
       conflicts=find_probe_conflicts(probes=probes, pos=pos, angs=angs)
       nconflicts=dim(conflicts)[1]
       if (nconflicts >0){
-        withTimeout(assign('newangs',MinimizeConflicts(pos=pos, angs=angs, cegs=cegs, delta_ang=delta_ang, probe_conflicts=conflicts)), timeout=timeout, onTimeout='warning')
+        try(withTimeout(assign('newangs',MinimizeConflicts(pos=pos, angs=angs, cegs=cegs, delta_ang=delta_ang, probe_conflicts=conflicts)), timeout=timeout, onTimeout=timeout_behaviour))
       }
       if (FALSE %in% (angs==newangs)){
         angs=newangs
@@ -686,7 +707,7 @@ wrapMinimizeConflicts <- function(pos=pos, angs=angs, cegs=cegs, init_delta_ang=
       print('Could not find a solution with delta_ang=20, trying delta_ang=45')
       delta_ang=45
       newangs=angs
-      withTimeout(assign('newangs',MinimizeConflicts(pos=pos, angs=angs, cegs=cegs, delta_ang=delta_ang, probe_conflicts=conflicts)), timeout=timeout, onTimeout='warning')
+      try(withTimeout(assign('newangs',MinimizeConflicts(pos=pos, angs=angs, cegs=cegs, delta_ang=delta_ang, probe_conflicts=conflicts)), timeout=timeout, onTimeout=timeout_behaviour))
       if (FALSE %in% (angs==newangs)){
         angs=newangs
       } else {
@@ -704,7 +725,7 @@ wrapMinimizeConflicts <- function(pos=pos, angs=angs, cegs=cegs, init_delta_ang=
       print('Could not find a solution with delta_ang=45, trying delta_ang=90')
       delta_ang=90
       newangs=angs
-      withTimeout(assign('newangs',MinimizeConflicts(pos=pos, angs=angs, cegs=cegs, delta_ang=delta_ang, probe_conflicts=conflicts)), timeout=timeout, onTimeout='warning')
+      try(withTimeout(assign('newangs',MinimizeConflicts(pos=pos, angs=angs, cegs=cegs, delta_ang=delta_ang, probe_conflicts=conflicts)), timeout=timeout, onTimeout=timeout_behaviour))
       if (FALSE %in% (angs==newangs)){
         angs=newangs
       } else {
@@ -783,9 +804,10 @@ MinimizeGuideConflicts <- function(pos=pos, angs=angs, cegs=cegs, gpos=gpos, gan
 
 compute_total_conflict_area <- function(angs,pos=pos){
   probes=defineprobe(x=pos[,'x'],y=pos[,'y'],angs=angs, interactive=FALSE)
+  lpoly=length(probes[,1]) #length of polygon defining probe.
   sprobes=data.frame()
   for (i in c(1:(length(pos[,'x'])))) {
-    sprobes <- rbind(sprobes,cbind(PID=rep(i, 12), POS=1:12, X=probes[,paste('X',as.character(i), sep='')], Y=probes[,paste('Y',as.character(i), sep='')]))
+    sprobes <- rbind(sprobes,cbind(PID=rep(i, lpoly), POS=1:lpoly, X=probes[,paste('X',as.character(i), sep='')], Y=probes[,paste('Y',as.character(i), sep='')]))
   }
   sprobes=as.PolySet(sprobes)
   junction=joinPolys(sprobes,sprobes,operation='INT')
@@ -806,11 +828,12 @@ cleanconflictedstds <- function(angs,pos=pos, stdpos=pos_master$stdpos){
   draw_all(x=pos_master$pos[,'x'],y=pos_master$pos[,'y'],xs=stdpos[,'x'],ys=stdpos[,'y'],xg=pos_master$guidepos[,'x'],yg=pos_master$guidepos[,'y'], interactive=visualise)
   
   probes=defineprobe(x=pos[,'x'],y=pos[,'y'],angs=angs, interactive=visualise)
+  lpoly=length(probes[,1]) #length of polygon defining probe.
   
   #Setting up into sp objects that package rgeos will understand.
   sprobes=data.frame()
   for (i in c(1:ngalprobes)) {
-    sprobes <- rbind(sprobes,cbind(PID=rep(i, 12), POS=1:12, X=probes[,paste('X',as.character(i), sep='')], Y=probes[,paste('Y',as.character(i), sep='')]))
+    sprobes <- rbind(sprobes,cbind(PID=rep(i, lpoly), POS=1:lpoly, X=probes[,paste('X',as.character(i), sep='')], Y=probes[,paste('Y',as.character(i), sep='')]))
   }
   sprobes=as.PolySet(sprobes)
   for (j in c(1:ngalprobes)) {
@@ -834,11 +857,12 @@ cleanconflictedguides <- function(angs,pos=pos, guidepos=pos_master$guidepos){
   draw_all(x=pos_master$pos[,'x'],y=pos_master$pos[,'y'],xs=pos_master$stdpos[,'x'],ys=pos_master$stdpos[,'y'],xg=pos_master$guidepos[,'x'],yg=pos_master$guidepos[,'y'], interactive=visualise)
   
   probes=defineprobe(x=pos[,'x'],y=pos[,'y'],angs=angs, interactive=visualise)
+  lpoly=length(probes[,1]) #length of polygon defining probe.
   
   #Setting up into sp objects that package rgeos will understand.
   sprobes=data.frame()
   for (i in c(1:(ngalprobes+nstdprobes))) {
-    sprobes <- rbind(sprobes,cbind(PID=rep(i, 12), POS=1:12, X=probes[,paste('X',as.character(i), sep='')], Y=probes[,paste('Y',as.character(i), sep='')]))
+    sprobes <- rbind(sprobes,cbind(PID=rep(i, lpoly), POS=1:lpoly, X=probes[,paste('X',as.character(i), sep='')], Y=probes[,paste('Y',as.character(i), sep='')]))
   }
   sprobes=as.PolySet(sprobes)
   for (j in c(1:(ngalprobes+nstdprobes))) {
@@ -861,41 +885,70 @@ cleanconflictedguides <- function(angs,pos=pos, guidepos=pos_master$guidepos){
   
   #Dividing into quadrants:
   first_q=clguidepos[clguidepos[,'x']<0 & clguidepos[,'y']<0,]
-  print(first_q)
+  #print(first_q)
   second_q=clguidepos[clguidepos[,'x']<0 & clguidepos[,'y']>=0,]
-  print(second_q)
+  #print(second_q)
   third_q=clguidepos[clguidepos[,'x']>=0 & clguidepos[,'y']>=0,]
-  print(third_q)
+  #print(third_q)
   fourth_q=clguidepos[clguidepos[,'x']>=0 & clguidepos[,'y']<0,]
-  print(fourth_q)
+  #print(fourth_q)
   
   oclguidepos=data.frame(x=double(),y=double()) #ordered and cleaned guide positions.
-  
   #Cycling through by quadrant to ensure guides from all corners of the plate get prioritised:
   gui=1
+  print(gui)
+  
+  
+  remaining_first_q = nrow(as.matrix(first_q))
+  remaining_second_q = nrow(as.matrix(second_q))
+  remaining_third_q = nrow(as.matrix(third_q))
+  remaining_fourth_q = nrow(as.matrix(fourth_q))
+                            
   while ((length(oclguidepos[,1]) < length(clguidepos[,1])) & gui < length(clguidepos[,1])){
-    if (length(as.matrix(first_q))>2){
-      try(assign('oclguidepos',rbind(oclguidepos, first_q[gui,])), TRUE)
+    
+    if (remaining_first_q > 0) {
+      if (nrow(as.matrix(first_q))>1){
+        assign('oclguidepos',rbind(oclguidepos, first_q[gui,]))
+        remaining_first_q = remaining_first_q -1
     } else{
-      try(assign('oclguidepos',rbind(oclguidepos, as.vector(first_q))), TRUE)
+      assign('oclguidepos',rbind(oclguidepos, as.vector(first_q)))
+      remaining_first_q = remaining_first_q -1
     }
-    if (length(as.matrix(second_q))>2){
-      try(assign('oclguidepos',rbind(oclguidepos, second_q[gui,])), TRUE)
+      
+    }
+    if (remaining_second_q > 0){
+      
+    if (nrow(as.matrix(second_q))>1 & remaining_second_q > 0){
+      assign('oclguidepos',rbind(oclguidepos, second_q[gui,]))
+      remaining_second_q = remaining_second_q -1
     } else {
-      try(assign('oclguidepos',rbind(oclguidepos, as.vector(second_q))), TRUE)
+      assign('oclguidepos',rbind(oclguidepos, as.vector(second_q)))
+      remaining_second_q = remaining_second_q - 1
     }
-    if (length(as.matrix(third_q))>2){
-      try(assign('oclguidepos',rbind(oclguidepos, third_q[gui,])), TRUE)
+    }
+    
+    if (remaining_third_q > 0){
+    if (nrow(as.matrix(third_q))>1){
+      assign('oclguidepos',rbind(oclguidepos, third_q[gui,]))
+      remaining_third_q = remaining_third_q -1
     } else {
-      try(assign('oclguidepos',rbind(oclguidepos, as.vector(third_q))), TRUE)
+      assign('oclguidepos',rbind(oclguidepos, as.vector(third_q)))
+      remaining_third_q = remaining_third_q -1
     }
-    if (length(as.matrix(fourth_q))>2){
-      try(assign('oclguidepos',rbind(oclguidepos, fourth_q[gui,])), TRUE)
+    }
+    if (remaining_fourth_q > 0){
+    if (nrow(as.matrix(fourth_q))>1){
+      assign('oclguidepos',rbind(oclguidepos, fourth_q[gui,]))
+      remaining_fourth_q = remaining_fourth_q -1
     } else{
-      try(assign('oclguidepos',rbind(oclguidepos, as.vector(fourth_q))), TRUE)
+      assign('oclguidepos',rbind(oclguidepos, as.vector(fourth_q)))
+      remaining_fourth_q = remaining_fourth_q -1
+    }
     }
     gui=gui+1
+    #print(c('oclguidepos is length', length(oclguidepos[,1])))
   }
+  #print(c("OCLguidepos has ", nrow(oclguidepos), "rows"))
   colnames(oclguidepos)=c('x','y')
   
   return(oclguidepos)
@@ -1074,10 +1127,10 @@ configure_stdstars <- function(pos, angs, cegs, stdpos){
     nconflicts = dim(tempconflicts)[1]
     
     if (nconflicts > 0) {
-      print(tempos)
+      #print(tempos)
       #print(tempangs)
-      print(tempconflicts)
-      withTimeout(expr=assign('tempangs',MinimizeConflicts(pos=tempos, angs=tempangs, cegs=tempcegs, probe_conflicts=tempconflicts)), timeout=timeout, onTimeout='warning')
+      #print(tempconflicts)
+      try(withTimeout(expr=assign('tempangs',MinimizeConflicts(pos=tempos, angs=tempangs, cegs=tempcegs, probe_conflicts=tempconflicts)), timeout=timeout, onTimeout=timeout_behaviour))
 
       draw_all(x=pos_master$pos[,'x'],y=pos_master$pos[,'y'],xs=clstdpos[,'x'],ys=clstdpos[,'y'],xg=pos_master$guidepos[,'x'],yg=pos_master$guidepos[,'y'], interactive=visualise)
       temprobes = defineprobe(x=tempos[,'x'],y=tempos[,'y'],angs=tempangs, interactive=visualise)
@@ -1119,7 +1172,7 @@ configure_guidestars <- function(pos, angs, cegs, guidepos){ #In this case, pos
   
   #* Added by Sam- this seems to fix the case where cleanconflictedguides returns a list with NAs in it
   clguidepos=clguidepos[!is.na(clguidepos)[,1],]
-  if (lengths(clguidepos)[1] < 3) {
+  if (lengths(clguidepos)[1] < 6) {
     print(paste('**Seem to only have ', lengths(clguidepos)[1], ' un-clashing guide stars in this field! Exiting and trying again...', sep=''))
     stdflag='GuideFail'
     #* Added by Sam so we can catch when we error and when we just can't configure
@@ -1130,7 +1183,7 @@ configure_guidestars <- function(pos, angs, cegs, guidepos){ #In this case, pos
   draw_all(x=pos_master$pos[,'x'],y=pos_master$pos[,'y'],xs=pos_master$stdpos[,'x'],ys=pos_master$stdpos[,'y'],xg=clguidepos[,'x'],yg=clguidepos[,'y'], interactive=visualise)
   probes=defineprobe(x=pos[,'x'],y=pos[,'y'],angs=angs, interactive=visualise)
   
-  #Adding 3-6 guides probes:
+  #Adding 6 guides probes:
   cg=0 #This will keep track of the number of configured guides so far.
   sel_guides=c() #This keeps track of selectec guide star probe numbers.
   gui=1 #This will cycle through all available guide stars.
@@ -1158,7 +1211,7 @@ configure_guidestars <- function(pos, angs, cegs, guidepos){ #In this case, pos
     
     if (nconflicts > 0) {
       newgangs=tempgangs
-      withTimeout(expr=assign('newgangs',MinimizeGuideConflicts(pos=pos, angs=angs, cegs=cegs, gpos=tempgpos, gangs=tempgangs, gcegs=gcegs, guide_conflicts=tempconflicts)), timeout=timeout, onTimeout='warning')
+      try(withTimeout(expr=assign('newgangs',MinimizeGuideConflicts(pos=pos, angs=angs, cegs=cegs, gpos=tempgpos, gangs=tempgangs, gcegs=gcegs, guide_conflicts=tempconflicts)), timeout=timeout, onTimeout=timeout_behaviour))
       tempgprobes=defineguideprobe(gxs=tempgpos[,'x'],gys=tempgpos[,'y'],gangs=newgangs, interactive=visualise)
       tempconflicts=find_guide_conflicts(probes=probes, pos=pos, angs=angs, gprobes=tempgprobes, gpos=tempgpos, gangs=newgangs)
       nconflicts=dim(tempconflicts)[1]
@@ -1179,8 +1232,8 @@ configure_guidestars <- function(pos, angs, cegs, guidepos){ #In this case, pos
     }
     gui = gui+1
   }
-  
-  if (cg < 3) {
+  print(paste('Configured ',as.character(cg), ' guide stars in this field'), sep='')
+  if (cg < 6) {
     print(paste('**Could only configure ',as.character(cg), ' guide stars in this field. Boo! :-('), sep='')
     stdflag='GuideFail'
   }
@@ -1284,7 +1337,7 @@ configure_HECTOR <- function(pos_master=pos_master){
   angs=withstdstars$angs
   stdflag=withstdstars$stdflag
   cegs=withstdstars$cegs
-  print('19 galaxy probes and 2 standard probes are in! Let\'s put some standard guide probes!')
+  print('19 galaxy probes and 2 standard probes are in! Let\'s put some guide probes!')
   
   #Configure 3-6 guide stars.
   guides=configure_guidestars(pos=pos, angs=angs, cegs=cegs, guidepos=pos_master$guidepos)
