@@ -1,5 +1,6 @@
 from astropy.io import fits 
 from astropy.table import Table
+import numpy as np
 
 def load_FITS_table_in_pandas(fname):
 
@@ -10,4 +11,5 @@ def load_FITS_table_in_pandas(fname):
 def save_dataframe_as_FITS_table(df, fname, overwrite=True):
 
     t=Table.from_pandas(df)
+    t = t.filled(fill_value=np.nan)
     return t.write(fname, overwrite=overwrite)
\ No newline at end of file
